@@ -6,7 +6,7 @@
     Contains SphinxError and a few subclasses (in an extra module to avoid
     circular import problems).
 
-    :copyright: Copyright 2007-2009 by the Sphinx team, see AUTHORS.
+    :copyright: Copyright 2007-2010 by the Sphinx team, see AUTHORS.
     :license: BSD, see LICENSE for details.
 """
 
@@ -28,7 +28,7 @@ class ExtensionError(SphinxError):
     category = 'Extension error'
 
     def __init__(self, message, orig_exc=None):
-        super(ExtensionError, self).__init__(message)
+        SphinxError.__init__(self, message)
         self.orig_exc = orig_exc
 
     def __repr__(self):
@@ -38,11 +38,27 @@ class ExtensionError(SphinxError):
         return '%s(%r)' % (self.__class__.__name__, self.message)
 
     def __str__(self):
-        parent_str = super(ExtensionError, self).__str__()
+        parent_str = SphinxError.__str__(self)
         if self.orig_exc:
             return '%s (exception: %s)' % (parent_str, self.orig_exc)
         return parent_str
 
 
+class ConfigError(SphinxError):
+    category = 'Configuration error'
+
+
 class ThemeError(SphinxError):
     category = 'Theme error'
+
+
+class VersionRequirementError(SphinxError):
+    category = 'Sphinx version error'
+
+
+class PycodeError(Exception):
+    def __str__(self):
+        res = self.args[0]
+        if len(self.args) > 1:
+            res += ' (exception was: %r)' % self.args[1]
+        return res
